@@ -26,6 +26,7 @@ class GpuCommand(PluginCommand):
                 gpu processes
                 gpu system
                 gpu status
+                gpu count
                 gpu
 
           This command returns some information about NVIDIA GPUs if your 
@@ -76,10 +77,13 @@ class GpuCommand(PluginCommand):
                 arguments.pretty = True
                 result = gpu.status()
 
+            elif arguments.count:
+                arguments.pretty = True
+                result = gpu.count
+
             else:
                 result = gpu.smi()
 
-
             try:
                 if arguments.pretty:
                     result = json.dumps(result, indent=2)
