@@ -1,12 +1,30 @@
+import collections
+
 import xmltodict
 from cloudmesh.common.Shell import Shell
+#from cloudmesh.common.Printer import Printer
+import pprint
 import os
 import yaml
 
+
 class Gpu:
 
     def __init__(self):
-        pass
+        try:
+            self._smi = dict(self.smi(output="json"))['nvidia_smi_log']['gpu']
+            if not isinstance(self._smi, list):
+                self._smi = [self._smi]
+        except KeyError:
+            raise RuntimeError("nvidia-smi not installed.")
+
+    @property
+    def count(self):
+        try:
+            number = int(Shell.run("nvidia-smi --list-gpus | wc -l").strip())
+        except:
+            number = 0
+        return number
 
     def vendor(self):
         if os.name != "nt":
@@ -23,19 +41,22 @@ class Gpu:
                 results = None
         return result
 
-
     def processes(self):
+        result = None
         try:
-            result = dict(self.smi(output="json"))
-            result = result["nvidia_smi_log"]["gpu"]["processes"]["process_info"]
-        except:
-            result = None
+            # We want to call this each time, as we want the current processes
+            result = dict(self.smi(self, output="json"))["nvidia_smi_log"]["gpu"]
+            if isinstance(result, list):
+                result = [x['processes']['process_info'] for x in result]
+            else:
+                result = result["processes"]["process_info"]
+        except KeyError:
+            pass
         return result
 
     def system(self):
-        try:
-            result = dict(self.smi(output="json"))
-            result = result["nvidia_smi_log"]["gpu"]
+        result = self._smi
+        for gpu_instance in range(len(self._smi)):
             for attribute in [
                     '@id',
                     #'product_name',
@@ -89,16 +110,16 @@ class Gpu:
                     'supported_clocks',
                     'processes'
                     ]:
-                del result[attribute]
-                result["vendor"] = self.vendor()
-        except:
-            result = None
+                try:
+                    del result[gpu_instance][attribute]
+                    result[gpu_instance]["vendor"] = self.vendor()
+                except KeyError:
+                    pass
         return result
 
     def status(self):
-        try:
-            result = dict(self.smi(output="json"))
-            result = result["nvidia_smi_log"]["gpu"]
+        result = self._smi
+        for gpu_instance in range(len(self._smi)):
             for attribute in [
                     '@id',
                     'product_name',
@@ -152,9 +173,10 @@ class Gpu:
                     'supported_clocks',
                     'processes'
                     ]:
-                del result[attribute]
-        except:
-            result = None
+                try:
+                    del result[gpu_instance][attribute]
+                except KeyError:
+                    pass
         return result
 
     def smi(self, output=None):
@@ -162,8 +184,8 @@ class Gpu:
         # json
         # yaml
         try:
-            if output == None:
-                result = Shell.run("nvidia-smi")
+            if output is None:
+                result = Shell.run("nvidia-smi").replace("\r", "")
             else:
                 r = Shell.run("nvidia-smi -q -x")
                 if output == "xml":
@@ -176,3 +198,5 @@ class Gpu:
             result = None
         return result
 
+    def __str__(self):
+        return pprint.pformat(self._smi, indent=2)
